## R code for calculating standard errors
library(RColorBrewer)
library(quantreg)
library(ggplot2)
library(data.table)
library(Rcpp)

dir.name <- "baseline-pc3"

#outDir <- "figures/3types/"
#paramfile <- "../estimation/baseline-pc3/smmEstCMA_2014-09-07_23.30.29.txt"
#cfgFile <- "../estimation/baseline-pc3/baseline3.cfg"

if (!exists("outDir") || is.na(outDir)) {
  outDir <- paste("figures/",dir.name,"/",sep="")
}
est.dir <- paste("../estimation/",dir.name,sep="")
if (!exists("paramfile") || is.na(paramfile)) {
  est.dir <- paste("../estimation/",dir.name,sep="")
  pfiles <- paste(est.dir,dir(est.dir,pattern="smmEstCMA(.+).txt"),sep="/")
  paramfile <- pfiles[order(pfiles,decreasing=TRUE)[1]]
  warning(sprintf("Using %s as paramfile",paramfile))
}
if (!exists("cfgFile") || is.na(cfgFile)) {
  est.dir <- paste("../estimation/",dir.name,sep="")
  cfgFile <- paste(est.dir,dir(est.dir,pattern="(.+).cfg$"),sep="/")
  if (length(cfgFile)>1) {
    stop("Cannot determine configuration file")
  }
}
#outDir <- "figures/nogapx/"


## Settings
#datafile <- "../c/data/p20.txt"
datafile <- "../estimation/data/medD-full-const.sl3"
nplanUse <- 500
probKeep <- 0.1
probKeep65 <- 0.1

#paramfile <- "../estimation/6types/smmEstCMA_2014-08-07_22.55.25.txt"
#cfgFile <- "../estimation/6types/6types.cfg"
#outDir <- "figures/6types/"
#cfgFile <- "../estimation/termVexcessJan/termVexcessJan.cfg"
#paramfile <- "../estimation/termVexcessJan/smmEstCMA_2014-08-23_12.47.20.txt"#smmEstCMA_2014-05-31_08.01.08.txt"
#cfgFile <- "../estimation/noCovariates/noCovariatess.cfg"
#paramfile <- "../estimation/noCovariates/smmEstCMA_2014-08-24_20.19.03.txt"
#outDir <- "figures/noCovariates/"
fdims <-1.5*c(5.03937, 3.77953) ## figure dimensions, =128mm x 96mm = dimensions of beamer slide
nsim <- 1 ## simulations per observation
nbin <- 75
colors <- brewer.pal(8,"Dark2") ## colors for figures
colors <- colors[c(1:3,5,8)]
forceFig <- TRUE ## recreate figures even if they already exist?


## read configuration
cfg.txt <- readLines(cfgFile)
cfg.txt <- gsub("true","TRUE",cfg.txt)
cfg.txt <- gsub("false","FALSE",cfg.txt)
cfg <- new.env()
eval(parse(text=cfg.txt),envir=cfg)

NX <- cfg$NX + 1
raEst <- cfg$estRiskAversion
nmix <- cfg$NMIX
nlambda <- cfg$nLambda
singleLambdaRatio <- cfg$singleLambdaRatio
terminalV <- cfg$terminalV

require(MASS)
require(Matrix)
source("param.R")

if (!exists("result")) {
  dx <- 1e-3
  ## run simulations using C++ library
  cflib <- "cf.so"
  if (is.loaded("cfsim")) dyn.unload(cflib)
  dyn.load(cflib)
  
  tmp.file <- paste("x",as.numeric(Sys.time()),".tmp",sep="")
  pm <- parameters(paramfile,tmp.file)
  args <- list(datafile,tmp.file,dx, cfgFile, nsim)
  names(args) <- c("datafile","xfile","dx","cfgFile","nsim")
  result <- .Call("asymVar",args)
  x <- scan(tmp.file)
  nx <- length(x)
  unlink(tmp.file)
} else {
  warning("Simulation result already exists, not re-running.")
}

nm <- sqrt(length(result$W))
moment <- array(result$fval,dim=c(5,nm,nx))
W <- matrix(result$W,nrow=nm)
V <- matrix(result$vm,nrow=nm)
df <- matrix(result$df,nrow=nm,ncol=nx)

avar1 <- ginv(t(df) %*% W %*% df) %*% (t(df) %*% W %*% ((1+1/nsim) * V)
                                       %*% W %*% df) %*% ginv(t(df) %*% W %*% df)
rankMatrix(t(df) %*% W %*% df)

df2 <- df
for (i in 1:length(x)) {
  df2[,i] <- (moment[2,,i]-moment[3,,i])/(2*dx*x[i])
}
avar2 <- ginv(t(df2) %*% W %*% df2) %*% (t(df2) %*% W %*% ((1+1/nsim) *V)
                                         %*% W %*% df2) %*% ginv(t(df2) %*% W %*% df2)
rankMatrix(t(df2) %*% W %*% df2)

## table of parameter estimates and standard errors
dx <- 1e-1
dp <- sapply(1:length(x),function(i) {
  xdx <- x
  xdx[i] <- x[i]+dx
  pp <- unlist(parameters(x.in=xdx))
  xdx[i] <- x[i]-dx
  pm <- unlist(parameters(x.in=xdx))
  return((pp-pm)/(2*dx))
})
v1 <- dp%*%avar1%*%t(dp)
v2 <- dp%*%avar2%*%t(dp)

N <- 430921
param.var <- v2/N 
se <- list()
sig <- sqrt(diag(param.var))
for (n in names(pm) ) {
  se[[n]] <- sig[grep(n,names(sig))]
  dim(se[[n]]) <- dim(pm[[n]])
}
se$delta <- se$delta[1]

bnames <- c("0","RS","65")
## reorder types by total spending

nd <- 3
spend.full.ins <- 52* (pm$lam %*% pm$marginalProb) *exp(pm$mu + pm$sig^2/2)
ord <- order(spend.full.ins)

filename <- paste(outDir,"paramSE.tex",sep="")
#filename <- "tmp.tex"
cat("$\\delta$ & ",format(pm$delta,digits=nd)," \\\\ \n",file=filename, append=FALSE)
cat("& (", format(se$delta,digits=nd),") \\\\ \n", file=filename, append=TRUE)
if (raEst) {
  cat("& risk aversion & ",format(pm$riskAversion,digits=nd), file=filename,append=TRUE)
  cat("& (", format(se$riskAversion,digits=nd),") \\\\ \n", file=filename, append=TRUE)
}
cat(" \\\\ \n", file=filename,append=TRUE)
if (terminalV) {
  cat("& $\\delta_\\omega$ & ",format(pm$deltaOmega,digits=nd), file=filename,append=TRUE)
  cat("& (", format(se$deltaOmega,digits=nd),") \\\\ \n", file=filename, append=TRUE)
  cat("& $\\delta_\\theta$ & ",format(pm$deltaTheta,,digits=nd), file=filename,append=TRUE)
  cat("& (", format(se$deltaTheta,digits=nd),") \\\\ \n", file=filename, append=TRUE)
  cat(" \\\\ \n", file=filename,append=TRUE)  
}
cat(" ",sprintf("$j=%i$",1:nmix),sep=" & ", file=filename, append=TRUE)
cat(" \\\\ \n",file=filename, append=TRUE)

for(j in 1:ncol(pm$bmix)) {
  cat(sprintf("$\\beta_{%s}$",bnames[j]),format(pm$bmix[ord,j],digits=nd),sep=" & ", file=filename, append=TRUE)
  cat(" \\\\ \n", file=filename, append=TRUE)
  cat(" ",paste("(",format(se$bmix[ord,j],digits=nd),")",sep=""),sep=" & ", file=filename, append=TRUE)
  cat(" \\\\ \n", file=filename, append=TRUE)
}
cat("$\\mu$",format(pm$mu[ord],digits=nd),sep=" & ", file=filename, append=TRUE)
cat(" \\\\ \n", file=filename, append=TRUE)
cat(" ",paste("(",format(se$mu[ord],digits=nd),")",sep=""),sep=" & ", file=filename, append=TRUE)
cat(" \\\\ \n", file=filename, append=TRUE)

cat("$\\sigma$",format(pm$sig[ord],digits=nd),sep=" & ", file=filename, append=TRUE)
 cat(" \\\\ \n", file=filename, append=TRUE)
cat(" ",paste("(",format(se$sig[ord],digits=nd),")",sep=""),sep=" & ", file=filename, append=TRUE)
cat(" \\\\ \n", file=filename, append=TRUE)

cat("$p_\\omega$",format(pm$pom[ord],digits=nd),sep=" & ", file=filename, append=TRUE)
cat(" \\\\ \n", file=filename, append=TRUE)
cat(" ",paste("(",format(se$pom[ord],digits=nd),")",sep=""),sep=" & ", file=filename, append=TRUE)
cat(" \\\\ \n", file=filename, append=TRUE)


for (i in 1:nlambda) {
  cat("$\\lambda$",format(pm$lam[ord,i],digits=nd),sep=" & ",file=filename,append=TRUE) 
  cat(" \\\\ \n",file=filename,append=TRUE)
  cat(" ",paste("(",format(se$lam[ord,i],digits=nd),")",sep=""),sep=" & ", file=filename, append=TRUE)
  cat(" \\\\ \n",file=filename,append=TRUE)
}
cat("\\\\ \n",file=filename,append=TRUE)
cat("\\multicolumn{6}{l}{$\\lambda$ transition probabilities} \\\\ \n",file=filename,append=TRUE)
for(i in 1:nlambda) {
  cat(" & " , format(pm$transitionProb[i,],digist=nd), sep=" & ", file=filename,append=TRUE)
  cat(" \\\\ \n",file=filename,append=TRUE)
  cat(" ",paste("(",format(se$transitionProb[i,],digits=nd),")",sep=""),sep=" & ", file=filename, append=TRUE)
  cat(" \\\\ \n",file=filename,append=TRUE)

}
cat("\\multicolumn{6}{l}{$\\lambda$ marginal probabilities} \\\\ \n",file=filename,append=TRUE)
for(i in 1:nlambda) {
  cat(" & " , format(pm$marginalProb[i],digist=nd), sep=" & ", file=filename,append=TRUE)
  cat(" ",paste("(",format(se$marginalProb[i],digits=nd),")",sep=""),sep=" & ", file=filename, append=TRUE)
  cat(" \\\\ \n",file=filename,append=TRUE)
}
cat("\\\\ \n",file=filename,append=TRUE)
cat("\\\\ \n", file=filename, append=TRUE)


## Influence of moments
influence <- ginv(t(df) %*% W %*% df) %*% (t(df) %*% W)
nmix <- 5
xnames <- c("const","RS","65")
rownames(influence) <- c(sprintf("\\mu_%d",1:nmix),
                         sprintf("\\sigma_%d",1:nmix),
                         sprintf("\\beta_{%d,%s}",
                                 sapply(2:nmix,function(x) { rep(x,length(xnames))} ),
                                 rep(xnames,nmix-1)),
                         sprintf("p_%d",1:nmix),
                         sprintf("\\lambda_%d",1:nmix),
                         sprintf("\\lambda ratio"),
                         sprintf("\\delta"),
                         sprintf("\\delta_\\omega"),
                         sprintf("transition prob %d",1:2))
cdfCut <- c(100, 250, 500, 1000, 1500, 2000, 3000, 4000 , 6000)
nEM <- 21
limEM <- 500
emCut <- seq(-500,500,length.out=(nEM+1))

colnames(influence) <- c("E[spend]","V(spend)","P(spend=0)",
                         "C(spend half1, spend half2)",
                         "C(spend,RS)",
                         "C(spend,65)",
                         sprintf("P(spend<%.0f)",cdfCut),
                         sprintf("P(%.1f < spend-ICL < %.1f)", emCut[1:nEM], emCut[2:(nEM+1)]),
                         rep(sprintf("timing"),12))


scaled.influence <- influence * sqrt(matrix(1/diag(avar1),ncol=1) %*% matrix(diag(V)*(1+1/nsim), nrow=1) )

filename <- paste(outDir,"influence.csv",sep="")
write.csv(influence, file=filename)
filename <- paste(outDir,"scaledInfluence.csv",sep="")
write.csv(scaled.influence, file=filename)



influence2 <-  ginv(t(df2) %*% W %*% df2) %*% (t(df2) %*% W)
rownames(influence2) <- rownames(influence)
colnames(influence2) <- colnames(influence)

W2 <- W
W2[37:42,37:42] <- W2[37:42,37:42]/1000
inf3 <- ginv(t(df) %*% W %*% df) %*% (t(df) %*% W)
rownames(inf3) <- rownames(influence)
colnames(inf3) <- colnames(influence)


## impact of timing moments
index <- 37:ncol(influence)
rowMeans(abs(scaled.influence[,index]))/rowMeans(abs(scaled.influence))
rowMeans(abs(influence[,index]))/rowMeans(abs(influence))

rowSums(abs(scaled.influence[,index]))/rowSums(abs(scaled.influence))
rowSums(abs(influence[,index]))/rowSums(abs(influence))


## impact of bunching moments
index <- 16:36
rowSums(abs(scaled.influence[,index]))/rowSums(abs(scaled.influence))
rowSums(abs(influence[,index]))/rowSums(abs(influence))

rowMeans(abs(scaled.influence[,index]))/rowMeans(abs(scaled.influence))
rowMeans(abs(influence[,index]))/rowMeans(abs(influence))


cbind(1:ncol(df),sapply(1:ncol(df), function(i)  rankMatrix(df[,c(-i,-22)])))
