rm(list=ls())
library(foreign)
library(xtable)
dataDirectory <- "/disk/agedisk5/medicare.work/afinkels-DUA22559/schrimpf"
##dat =read.dta("/disk/agedisk5/medicare.work/afinkels-DUA22559/schrimpf/Data/extract/FullSample.dta")
dat =read.dta("/disk/agedisk5/medicare.work/afinkels-DUA22559/schrimpf/Data/extract/65Sample2.dta")

## collapse to weekly data
dat$week <- as.integer( (as.numeric(dat$srvc_dt) -
                         as.numeric(as.Date(sprintf("%4d-01-01",dat$yr))))/7)
dat$week[dat$week==53] <- 52 ## make last week of year have 8 (or 9) days
dat$joinweek <- as.integer((as.numeric(as.Date(sprintf("%4d-%2d-01",dat$yr,dat$partd_joinmonth)))
                            - as.numeric(as.Date(sprintf("%4d-01-01",dat$yr))) ) / 7)
dat$costWk <- ave(dat$totalcst,dat$bene_id,dat$week,FUN=sum)
## correction for id's with no claims
if (sum(is.na(dat$totalcst))>0) {
  ## check assumption that totalcst = NA implies that id made no claims
  stopifnot(sum(is.na(dat$totalcst)) ==
            length(unique(dat$bene_id[is.na(dat$totalcst)])))
  dat$week[is.na(dat$totalcst)] <- 51
  dat$costWk[is.na(dat$totalcst)] <- 0
}

daysin <- as.numeric(dat$srvc_dt) - as.numeric(as.Date(sprintf("%4d-%2d-01",dat$yr,dat$partd_joinmonth)))
weeksin <- dat$week - dat$joinweek
icDays <- ave(dat$totalcst*(daysin<12*7 & daysin>=0), dat$bene_id,
              FUN=function(x) { sum(x,na.rm=TRUE) });
icWeeks <- ave(dat$totalcst*(weeksin<12 & weeksin>=0), dat$bene_id,
               FUN=function(x) { sum(x,na.rm=TRUE) });

icd <- cbind(dat[,c("bene_id","ded_amt","partd_joinmonth","joinweek")],icDays,icWeeks)
icd <- icd[!duplicated(icd),]

# compute means by joinmonth or quarter and plan group
jm <- icd$partd_joinmonth
jq <- jm
jq[jm<=4] <- 1
jq[jm>4 & jm<=7] <- 2
jq[jm>=8] <- 3
jm[icd$ded_amt==0] <- jm[icd$ded_amt==0]+100
jq[icd$ded_amt==0] <- jq[icd$ded_amt==0]+10
tapply(icd$icDays/12, jq, FUN=function(x) { mean(x,na.rm=TRUE) })
tapply(icd$icWeeks/12, jq, FUN=function(x) { mean(x,na.rm=TRUE) })

tapply(icd$icDays/12, jm, FUN=function(x) { mean(x,na.rm=TRUE) })
tapply(icd$icWeeks/12, jm, FUN=function(x) { mean(x,na.rm=TRUE) })
tapply(icd$icWeeks/12, jm, FUN=function(x) { 12*sum(!is.na(x)) })

