rm(list=ls())
library(foreign)
library(xtable)
dataDirectory <- "/disk/agedisk5/medicare.work/afinkels-DUA22559/schrimpf"
##dat =read.dta("/disk/agedisk5/medicare.work/afinkels-DUA22559/schrimpf/Data/extract/FullSample.dta")
dat =read.dta("/disk/agedisk5/medicare.work/afinkels-DUA22559/schrimpf/Data/extract/65Sample2.dta")
figureDirectory <- "figures"
tableDirectory <- "tables"

## data processing
dat$bnftphas <- factor(dat$bnftphas)
# sort by id, claim date
dat <- dat[order(dat$bene_id,dat$srvc_dt),]
## collapse to weekly data
dat$week <- as.integer( (as.numeric(dat$srvc_dt) -
                         as.numeric(as.Date(sprintf("%4d-01-01",dat$yr))))/7)
dat$week[dat$week==52] <- 51 ## make last week of year have 8 (or 9) days (we count from 0)
dat$joinweek <- as.integer((as.numeric(as.Date(sprintf("%4d-%2d-01",dat$yr,dat$partd_joinmonth)))
                            - as.numeric(as.Date(sprintf("%4d-01-01",dat$yr))) ) / 7)

dat$costWk <- ave(dat$totalcst,dat$bene_id,dat$week,FUN=sum)
dat$payWk <- ave(dat$ptpayamt,dat$bene_id,dat$week,FUN=sum)
## correction for id's with no claims
if (sum(is.na(dat$totalcst))>0) {
  ## check assumption that totalcst = NA implies that id made no claims
  stopifnot(sum(is.na(dat$totalcst)) ==
            length(unique(dat$bene_id[is.na(dat$totalcst)])))
  
  dat$week[is.na(dat$totalcst)] <- 51
  dat$costWk[is.na(dat$totalcst)] <- 0
  dat$payWk[is.na(dat$totalcst)] <- 0
}

wkdat <- dat[,c("bene_id","week","joinweek","partd_joinmonth","yr"
                ,"plan","ded_amt","icl_amt","oopt_amt","costWk","payWk",
                "plan_coin_ded","plan_coin_preicl","plan_coin_gap",
                "plan_coin_cat")]
wkdat$phase <- rep(NA,nrow(wkdat))
wkdat$phase[dat$bnftphas=="DD"] <- 1
wkdat$phase[dat$bnftphas=="DP"] <- 1.51
wkdat$phase[dat$bnftphas=="DI"] <- 2.51
wkdat$phase[dat$bnftphas=="DC"] <- 3.51
wkdat$phase[dat$bnftphas=="PP"] <- 2
wkdat$phase[dat$bnftphas=="PI"] <- 2.52
wkdat$phase[dat$bnftphas=="PC"] <- 3.52
wkdat$phase[dat$bnftphas=="II"] <- 3
wkdat$phase[dat$bnftphas=="IC"] <- 3.53
wkdat$phase[dat$bnftphas=="CC"] <- 4

wkdat$phase <- ave(wkdat$phase,wkdat$bene_id,wkdat$week,FUN=mean)

wkdat <- wkdat[!duplicated(wkdat) & !is.na(wkdat$partd_joinmonth), ]

wkdat <- wkdat[order(wkdat$bene_id,wkdat$week),]

cumsum.na <- function(x) { y <- x
                           y[is.na(x)] <- 0
                           cumsum(y)
                         }
wkdat$cumPay <- ave(wkdat$payWk,wkdat$bene_id, wkdat$yr,
                   FUN=cumsum.na)
wkdat$cumCost <- ave(wkdat$costWk,wkdat$bene_id,wkdat$yr,
                    FUN=cumsum.na)
wkdat$segment <- as.factor(1*(wkdat$cumCost<wkdat$ded_amt) +
                           2*(wkdat$cumCost>=wkdat$ded_amt & wkdat$cumCost<wkdat$icl_amt) +
                           3*(wkdat$cumCost>=wkdat$icl_amt & wkdat$cumPay<wkdat$oopt_amt) +
                           4*(wkdat$cumPay>=wkdat$oopt_amt))

## FIXME: why so many na's?
wkdat$plan_coin_ded[is.na(wkdat$plan_coin_ded)] <- 1
wkdat <- wkdat[!is.na(wkdat$plan_coin_preicl), ]
wkdat$plan_coin_gap[is.na(wkdat$plan_coin_gap)] <- 1
wkdat$plan_coin_cat[is.na(wkdat$plan_coin_cat)] <- 0.1

wkdat$phase[is.na(wkdat$phase)] <- 0
## convert plan ids to numeric
wkdat$id <- unclass(as.factor(wkdat$bene_id))
wkdat$planNumber <- unclass(as.factor(wkdat$plan))
wkdat <- wkdat[,names(wkdat)!="bene_id" & names(wkdat)!="plan"]

wkdat <- wkdat[order(wkdat$id,wkdat$week),]
write.csv(wkdat,paste(dataDirectory,"/full.csv",sep=""),quote=FALSE,row.names=FALSE)
save(wkdat,file=paste(dataDirectory,"/full.Rdata",sep=""), compress=TRUE)
